/*
 * Copyright (c) 2004 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.
 */

/**
 ** circleToPoly
 **
 ** To approximate a circle to a polygon (with segments tangent to the circle)
 **
 ** Auteur: Sara FLEURY
 ** Date: 11/15/01
 **
 **/


#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "h2mathLib.h"

static int convertCircleToBrokenLine(double length, 
				     double curvature,
				     double devMax, /* max deviation from the circle */
				     double *dTheta,  /* Gyration between 2 segments */
				     double *segLength); /* Length of a complet segment */

int main (int argc, char *argv[])

{
  int nbSegs, i;
  double x0, y0, R, e, x, y;
  double theta;
  double length, dtheta;

  if (argc != 5) {
    printf ("usage: circleToPoly <x-center> <y-center> <radius> <max dev>\n");
    return 0;
  }

  x0 = atof(argv[1]);
  y0 = atof(argv[2]);
  R = atof(argv[3]);
  e = atof(argv[4]);

  printf ("circleToPoly x %f y %f R %f e %f\n", x0, y0, R, e);

  /* Convert the arc of circle to a broken line */
  nbSegs = convertCircleToBrokenLine(2*PI*R, 1/R, e, &dtheta, &length);

  /* Record */
  theta = 2*PI;
  printf ("points: %d\n", nbSegs);
  for (i=0; i<nbSegs; i++) {
    x = x0 + R * cos(theta);
    y = y0 + R * sin(theta);
    theta -= dtheta;
    printf ("%f %f\n", x, y);
  }
  printf ("FinalPos: %f %f 0.0\n", x0, y0);
  printf ("Target: %f %f 0.0\n", x0, y0);

  return 0 ;
}

/************************************************************************
 *
 * convertCircleToBrokenLine - Comput the broken line circumpsribed to 
 *                             an arc of circle. 
 *
 * Description: Cuts the arc of cicle in regular angles in order to obtain a
 *              max deviation between the circle and the resulting broken line 
 *              smaller than the specified value.  The segments are tangent to 
 *              the circle at each angle step. The first and the last segment
 *              will have the half length of the others ones.
 *              
 *               
 * Return: The number of segments that replaced the circle
 */

static int convertCircleToBrokenLine(double length, 
				     double curvature,
				     double devMax,
				     double *dTheta,  /* Gyration between 2 segments */
				     double *segLength) /* Length of a complet segment */
{
  double theta;
  double nbSteps;
  double stepAngle;

  if (devMax == 0.)
    return -1;

  /* Agular variation on the circle */
  theta = length*curvature;

  /* Compute step angle according to the maximal deviation */
  stepAngle = 2*acos(1./(1. + fabs(devMax*curvature)));

  /* Minimal number of steps to have an angle smaller than ME_STEP_ANGLE */
  nbSteps = ceil(fabs(theta)/stepAngle);

  /* Angle portion on the circle */
  *dTheta = theta/nbSteps;
  
  /* Length of each segment */
  *segLength = SIGN(length) * fabs(2.*tan(*dTheta/2.)/curvature);

/*  printf ("ME convertCircleToBrokenLine: %2.0f x %5.4f = %5.4f rd  (dev %f m)\n", 
	  nbSteps, *dTheta, theta, (1./cos(*dTheta/2.) - 1.)/fabs(curvature));
	  */
  return(nbSteps+1);
}

